
USE [AdventureWorks]
go

CREATE TABLE sales (
  salesid INT PRIMARY KEY IDENTITY,
  product INT,
  quatity INT,
  amount  DECIMAL(7,2)
)
go

CREATE TABLE inventory (
  productid INT,
  quantity INT
)
go

DELETE inventory
INSERT INTO inventory VALUES(42, 20000)
go

-- 1. END OF DEFINING TABLES

-- Traditional transaction update sales and inventory
BEGIN TRANSACTION
	INSERT INTO sales VALUES(1, 42, 3.50);

	UPDATE	inventory 
	SET		quantity = quantity - 1
	WHERE	productid = 42;
COMMIT
go

SELECT * FROM inventory
go

-- 2. END TRADITIONAL TRANSACTION

-- Simplified sending procedure
CREATE PROCEDURE update_inventory(@quantity INT)
AS
DECLARE @h UNIQUEIDENTIFIER;
BEGIN TRY
	BEGIN DIALOG CONVERSATION @h
	FROM SERVICE sender
	TO SERVICE 'receiver'
	WITH ENCRYPTION = OFF;

	SEND ON CONVERSATION @h
	(@quantity);
END TRY
BEGIN CATCH
	PRINT ERROR_MESSAGE()
END CATCH
END CONVERSATION @h;
go

-- 3. END SENDING PROCEDURE

--DROP PROCEDURE decrement_inventory

CREATE PROCEDURE decrement_inventory
AS
DECLARE @c UNIQUEIDENTIFIER;
DECLARE @h UNIQUEIDENTIFIER;
DECLARE @m INT;
DECLARE @t NVARCHAR(128);

WHILE 1 = 1
BEGIN

BEGIN TRANSACTION
WAITFOR(
   GET CONVERSATION GROUP @c FROM receiveq),
   TIMEOUT 100 ;

IF @c IS NULL 
  BEGIN
   ROLLBACK TRANSACTION;
   BREAK;
  END

WHILE 1 = 1
BEGIN
	RECEIVE TOP(1) 
        @h=conversation_handle, 
        @t=message_type_name,
		@m=CAST(message_body as INT) FROM receiveq 
     WHERE conversation_group_id = @c;

    IF @@rowcount <> 1 
		BREAK;
    IF @t = N'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
    OR @t = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error' 

    BEGIN 
		END CONVERSATION @h;
        CONTINUE
    END
    -- Need error handling here
	UPDATE	inventory 
	SET		quantity = quantity - @m
	WHERE	productid = 42; 
END
COMMIT TRANSACTION;
END
go

-- 4. END ACTIVATION PROCEDURE

ALTER QUEUE receiveq WITH ACTIVATION 
  (STATUS=ON,
   PROCEDURE_NAME=decrement_inventory,
   MAX_QUEUE_READERS=5,
   EXECUTE AS SELF
  )
go

-- 5. TURN ON ACTIVATION

-- Rewritten transaction
SELECT * FROM inventory
go

DECLARE @i INT
SET @i = 0
WHILE @i < 100
 BEGIN
   BEGIN TRANSACTION
   INSERT INTO sales VALUES(1, 42, 3.50);
   EXECUTE update_inventory 1
   COMMIT
   SET @i = @i + 1
 END
go

-- 6. END UPDATED TRANSACTION CODE

SELECT * FROM inventory
go

-- 7. END CHECK INVENTORY

ALTER QUEUE receiveq WITH ACTIVATION 
  (STATUS=OFF,
   PROCEDURE_NAME=decrement_inventory,
   MAX_QUEUE_READERS=5,
   EXECUTE AS SELF
  )
go

-- 8. END OF TURNING OFF ACTIVATION

SELECT * FROM inventory
go

DECLARE @i INT
SET @i = 0
WHILE @i < 100
 BEGIN
   BEGIN TRANSACTION
   INSERT INTO sales VALUES(1, 42, 3.50);
   EXECUTE update_inventory 1
   COMMIT
   SET @i = @i + 1
 END
go

-- 9. END TRANSACTION WITH ACTIVATION OFF

SELECT * FROM inventory
go

-- 10. END CHECK INVENTORY AGAIN

ALTER QUEUE receiveq WITH ACTIVATION 
  (STATUS=ON,
   PROCEDURE_NAME=decrement_inventory,
   MAX_QUEUE_READERS=5,
   EXECUTE AS SELF
  )
go

-- 11. TURN ON ACTIVATION AGAIN

-- Check inventory levels as its processing
SELECT * FROM inventory
go

